{% extends "base.j2" %}

{% block containerclass %}-fluid{% endblock%}

{% block navbar %}{% include 'navbar.j2' %}{% endblock %}

{% block sortables %}

<div class="container-fluid">

{{ breadcrumbs }}
{% set pages = [
    ("ASOS", "/ASOS/current.phtml"),
    ("AWOS", "/AWOS/current.phtml"),
    ("CoCoRaHS", "/cocorahs/current.phtml"),
    ("COOP", "/COOP/current.phtml"),
    ("DCP", "/DCP/current.phtml"),
    ("DCP", "/DCP/current.phtml"),
    ("ISU Soil Moisture", "/agclimate/current.phtml"),
    ("Other", "/other/current.phtml"),
    ("SCAN", "/scan/current.phtml"),
    ("My Favorites", "/my/current.phtml"),
    ("By State", "/current/all.phtml"),
    ("RWIS", "/RWIS/current.phtml"),
    ("RWIS Surface", "/RWIS/currentSF.phtml"),
    ("RWIS Traffic", "/RWIS/traffic.phtml"),
    ("RWIS Soil", "/RWIS/soil.phtml"),
    ("By NWS WFO", "/nws/obs.php"),
    ("USCRN", "/current/uscrn.phtml"),
    ] %}

<table class="table table-striped table-condensed">
<thead>
<tr>
<th colspan="6" bgcolor="#CCCCCC">View Sortable Currents For Network</th></tr>
</thead>
<tbody><tr>

{% for item in pages %}
    {% if loop.index0 > 0 and loop.index0 % 6 == 0 %}
        </tr><tr>
    {% endif %}
    {% if item[0] == current_network %}
        <th class="info" ><a href="{{ item[1] }}">{{ item[0] }}</a></th>
    {% else %}
        <th><a href="{{ item[1] }}">{{ item[0] }}</a></th>
    {% endif %}
{% endfor %}

</tr></tbody>
</table>
</div>

{% endblock %}

{% block content %}{{ content }}{% endblock %}

{% block footer %}{% include 'footer.j2' %}{% endblock %}
