{% extends "base.j2" %}

{% block navbar %}{% include 'navbar.j2' %}{% endblock %}

{% block content %}
<main role="main" id="main-content">
<div class="container-fluid">
{{ content }}
</div>
</main>
{% endblock %}

{% block footer %}{% include 'footer.j2' %}{% endblock %}
