"""Python Utilities developed for/by Iowa Environmental Mesonet

Python is an important part of the Iowa Environmental Mesonet (IEM).  This
package is used by many parts of the IEM codebase and hopefully somewhat
useful to others!?!?
"""
from ._version import get_version  # noqa: E402

__version__ = get_version()
