<header>
<nav class="navbar-iastate">
	<div class="container">
		<!-- navbar-iastate-left -->
		<ul class="nav navbar-nav navbar-iastate-left">
			<li><a href='https://www.iastate.edu'>iastate.edu</a></li>
			<li class="dropdown dropdown-hover isu-index">
				<a href="https://www.iastate.edu/index/A" class="dropdown-toggle" data-toggle="" role="button" aria-haspopup="true" aria-expanded="false">Index</a>
				<ul class="dropdown-menu isu-index-alpha">
					<li><a href="https://www.iastate.edu/index/A/">A</a></li>
					<li><a href="https://www.iastate.edu/index/B/">B</a></li>
					<li><a href="https://www.iastate.edu/index/C/">C</a></li>
					<li><a href="https://www.iastate.edu/index/D/">D</a></li>
					<li><a href="https://www.iastate.edu/index/E/">E</a></li>
					<li><a href="https://www.iastate.edu/index/F/">F</a></li>
					<li><a href="https://www.iastate.edu/index/G/">G</a></li>
					<li><a href="https://www.iastate.edu/index/H/">H</a></li>
					<li><a href="https://www.iastate.edu/index/I/">I</a></li>
					<li><a href="https://www.iastate.edu/index/J/">J</a></li>
					<li><a href="https://www.iastate.edu/index/K/">K</a></li>
					<li><a href="https://www.iastate.edu/index/L/">L</a></li>
					<li><a href="https://www.iastate.edu/index/M/">M</a></li>
					<li><a href="https://www.iastate.edu/index/N/">N</a></li>
					<li><a href="https://www.iastate.edu/index/O/">O</a></li>
					<li><a href="https://www.iastate.edu/index/P/">P</a></li>
					<li><a href="https://www.iastate.edu/index/Q/">Q</a></li>
					<li><a href="https://www.iastate.edu/index/R/">R</a></li>
					<li><a href="https://www.iastate.edu/index/S/">S</a></li>
					<li><a href="https://www.iastate.edu/index/T/">T</a></li>
					<li><a href="https://www.iastate.edu/index/U/">U</a></li>
					<li><a href="https://www.iastate.edu/index/V/">V</a></li>
					<li><a href="https://www.iastate.edu/index/W/">W</a></li>
					<li><a href="https://www.iastate.edu/index/X/">X</a></li>
					<li><a href="https://www.iastate.edu/index/Y/">Y</a></li>
					<li><a href="https://www.iastate.edu/index/Z/">Z</a></li>
				</ul>
			</li>
		</ul>
		<!-- /navbar-iastate-left -->

		<!-- navbar-iastate-right -->
		<ul class="nav navbar-nav navbar-right navbar-iastate-right">
			<li><a href="https://info.iastate.edu/">Directory</a></li>
			<li><a href="https://www.fpm.iastate.edu/maps/">Maps</a></li>
			<li><a href="https://web.iastate.edu/safety/">Safety</a></li>
			<li class="dropdown dropdown-hover">
				<a href="//web.iastate.edu/signons/" class="dropdown-toggle" data-toggle="" role="button" aria-haspopup="true" aria-expanded="false">Sign Ons</a>
				<ul class="dropdown-menu">
					<li><a href="https://accessplus.iastate.edu/">AccessPlus</a></li>
					<li><a href="https://bb.its.iastate.edu/">Blackboard</a></li>
					<li><a href="https://iastate.box.com/">CyBox</a></li>
					<li><a href="https://cymail.iastate.edu/">CyMail</a></li>
					<li><a href="https://outlook.iastate.edu/">Outlook</a></li>
					<li><a href="https://web.iastate.edu/signons/">More Sign Ons...</a></li>
				</ul>
			</li>
		</ul>
		<!-- /navbar-iastate-right -->

	</div>
</nav>
<!-- /navbar-iastate -->

<nav class="navbar-site">
	<div class="container">

		<div class="navbar-site-wordmark">
			<a href="/" title="Home" class="wordmark-unit">
				<span class="wordmark-isu">Iowa State University</span>
				<span class="wordmark-unit-title">Iowa Environmental Mesonet</span>
			</a>
		</div>
 
		<div class="navbar-header visible-xs visible-sm">
			<button id="navbar-menu-button" type="button" class="navbar-toggle navbar-toggle-left collapsed" data-toggle="collapse" data-target="#navbar-menu-collapse, #navbar-site-links-collapse" aria-expanded="false">
				<span class="navbar-toggle-icon menu-icon">
					<span class="icon-bar"></span>
					<span class="icon-bar"></span>
					<span class="icon-bar"></span>
				</span>
				<span class="navbar-toggle-label">
					Menu <span class="sr-only">Toggle</span>
				</span>
			</button>
			<button id="navbar-search-button" type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar-site-search-collapse" aria-expanded="false">
				<span class="navbar-toggle-icon search-icon"></span>
				<span class="navbar-toggle-label">
					Search <span class="sr-only">Toggle</span>
				</span>
			</button>
		</div>

		<div class="navbar-site-info collapse navbar-collapse" id="navbar-site-search-collapse">
			<form action="https://www.google.com/search" class="navbar-site-search" method="GET" role="search">
				<label for="search-input" class="sr-only">Search</label>
				<input value="mesonet.agron.iastate.edu" name="sitesearch" type="hidden" />
				<input name="q" id="search-input" aria-label="Text input for search" title="Search" placeholder="Search" tabindex="0" type="text" class="form-control">
				<input class="hidden" title="Submit" type="submit" value="Search">
				<span class="search-icon"></span>
			</form>
		</div>

		<div class="navbar-site-info collapse navbar-collapse navbar-caps" id="navbar-site-links-collapse">
			<ul class="nav navbar-nav navbar-right">
				<!-- Site-wide links. Customized for your site -->
				<li><a href="/info/contacts.php">contact us</a></li>
				<li><a href="/disclaimer.php">disclaimer</a></li>
				<li><a href="/apps.php">apps</a></li>
			</ul>
		</div>

	</div>
</nav>
</header>

{% set pages = [
    ['archive', {
        "title": "Archive",
        "subs": [
    ("base", "Archive Mainpage", "/archive/"),
    ("schema", "Archive Schema" , "/archive/schema.php"),
    ("browse", "Browse data/" , "/archive/data/"),
    ("hrain", "Hourly Precip" , "/rainfall/obhour.phtml"),
    ("iemre", "IEM Reanalysis" , "/iemre/"),
    ("cases", "Interesting Cases" , "/cases/"),
    ("mos", "Model Output Statistics" , "/mos/"),
    ("afos", "NWS Text Products" , "/wx/afos/list.phtml"),
    ("pireps", "PIREPs" , "/request/gis/pireps.php"),
    ("smos", "Soil Moisture Satellite" , "/smos/"),
    ("raob", "Sounding Archive" , "/archive/raob/"),
    ("tm", "Time Machine" , "/timemachine/"),
        ],
    }],
    ['climatology', {
        "title": "Climate",
        "subs": [
    ("base", "Climate Mainpage", "/climate/"),
    ("extremes", "Daily Climatology", "/COOP/extremes.php"),
    ("main", "Mainpage", "/climate/"),
    ("climodat", "Climodat", "/climodat/"),
    ("climodatm", "Climodat Monitor", "/climodat/monitor.php"),
    ("drought", "Drought", "/dm/"),
    ("today", "Today", "/climate/today.phtml"),
    ("yesterday", "Yesterday", "/climate/yesterday.phtml"),
    ("week", "Week", "/climate/week.phtml"),
    ("month", "Month", "/climate/month.phtml"),
    ("gs", "Growing Season", "/climate/gs.phtml"),
    ("year", "Year", "/climate/year.phtml"),
        ],
    }],
    ['current', {
        "title": "Current",
        "subs": [
    ("base", "Current Mainpage", "/current/"),
    ("plotter", "Auto Data Plotter", "/plotting/auto/"),
    ("placefiles", "Gibson Ridge Placefiles", "/request/grx/"),
    ("gs", "Growing Season", "/current/gs.phtml"),
    ("jsonp", "JSON Webservices", "/json/"),
    ("afos", "NWS Text", "/wx/afos/"),
    ("products", "Products", "/current/"),
    ("radar", "RADAR & Satellite", "/current/radar.phtml"),
    ("sort", "Sortable Currents", "/my/current.phtml"),
        ],
    }],
    ['iem', {
        "title": "Info",
        "subs": [
    ("base", "Info Mainpage", "/info.php"),
    ("agweater", "Ag Weather/Climate Info", "/agweather/"),
    ("apps", "Application Index", "/apps.php"),
    ("feature", "Daily Features", "/onsite/features/past.php"),
    ("links", "Links", "/info/links.php"),
    ("networks", "Network Tables", "/sites/networks.php"),
    ("news", "News", "/onsite/news.phtml"),
    ("present", "Presentations", "/present/"),
    ("sites", "Station Data and Metadata", "/sites/locate.php"),
    ("qc", "Quality Control", "/QC/"),
    ("variables", "Variables", "/info/variables.phtml"),
        ],
    }],
    ['gis', {
        "title": "GIS",
        "subs": [
    ("base", "GIS Mainpage", "/GIS/"),
    ("browse", "Browse Data", "/data/gis/"),
    ("nexrad", "NEXRAD Mosaic", "/docs/nexrad_mosaic/"),
    ("ogc", "OGC Webservices", "/ogc/"),
    ("rainfall", "Rainfall Data", "/rainfall/"),
    ("radmap", "RadMap API", "/GIS/radmap_api.phtml"),
    ("radview", "RADAR Services", "/GIS/radview.phtml"),
    ("satellite", "Satellite Data", "/GIS/goes.phtml"),
        ],
    }],
    ['networks', {
        "title": "Networks",
        "subs": [
    ("base", "Networks Mainpage", "/"),
    ("asos", "ASOS", "/ASOS/"),
    ("awos", "AWOS", "/AWOS/"),
    ("cocorahs", "CoCoRaHS", "/cocorahs/"),
    ("coop", "NWS COOP", "/COOP/"),
    ("dcp", "DCP", "/DCP/"),
    ("agclimate", "ISU Soil Moisture", "/agclimate/"),
    ("flux", "NLAE Flux", "/nstl_flux/"),
    ("rwis", "RWIS", "/RWIS/"),
    ("scan", "SCAN", "/scan/"),
    ("schoolnet", "SchoolNet", "/schoolnet/"),
    ("other", "Other", "/other/"),
        ],
    }],
    ['roads', {
        "title": "Roads",
        "subs": [
    ("base", "Roads Mainpage", "/roads/"),
    ("gis", "GIS Products", "/roads/gis.phtml"),
    ("history", "Historical", "/roads/history.phtml"),
    ("maps", "Interactive Map", "/roads/maps.phtml"),
    ("sort", "Sortable Text", "/roads/rc.phtml"),
        ],
    }],
    ['severe', {
        "title": "Svr Wx",
        "subs": [
    ("base", "Severe Weather Mainpage", "/current/severe.phtml"),
    ("cow", "IEM Cow (SBW Verification)", "/cow/"),
    ("iembot", "iembot", "/projects/iembot/"),
    ("interact", "Interact Radar", "/GIS/apps/rview/warnings.phtml"),
    ("lsr", "Local Storm Report App", "/lsr/"),
    ("raccoon", "IEM Raccoon (SBW Powerpoints)", "/raccoon/"),
    ("river", "River Summary", "/river/"),
    ("search", "Search for Warnings", "/vtec/search.php"),
    ("spcoutlook", "SPC Convective Outlook / MCD Search", "/nws/spc_outlook_search/"),
    ("watch", "SPC Watches", "/GIS/apps/rview/watch.phtml"),
    ("vtec", "VTEC Browser", "/vtec/"),
        ],
    }],
    ['webcam', {
        "title": "Webcams",
        "subs": [
    ("base", "Webcam mainpage", "/projects/webcam.php"),
    ("loop", "Build your own lapses", "/current/bloop.phtml"),
    ("cool", "Cool lapses", "/cool/"),
    ("viewer", "IEM Webcam Viewer", "/current/viewer.phtml"),
    ("isu", "ISU Campus Webcams", "/current/isucams.phtml"),
    ("lapse", "Recent lapses", "/current/camlapse/"),
    ("stills", "Still images", "/current/webcam.php"),
        ],
    }],
] %}

<div class="container">
<div class="navbar navbar-default">
    <div class="navbar-header">
        <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
        </button>
        </div>
    <div class="navbar-collapse collapse">
        <div class="hidden-xs hidden-sm">
        <a href="/"><img style="height: 50px;" alt="IEM" class="img-responsive pull-left" src="/images/logo_small.png" /></a>
                </div>

<ul class="nav navbar-nav">
        
<li class="dropdown"><a class="dropdown-toggle" data-toggle="dropdown" 
        href="#"> <span class="fa fa-home"></span> </a>
        <ul class="dropdown-menu">
                <li><a href="https://mesonet.agron.iastate.edu">Iowa Environmental Mesonet</a></li>
            <li><a href="http://iowa.cocorahs.org">Iowa CoCoRaHS</a></li>
                <li><a href="https://dailyerosion.org">Daily Erosion Project</a></li>
                <li><a href="https://datateam.agron.iastate.edu/cscap/">Sustainable Corn</a></li>
                <li><a href="https://weather.im">Weather.IM Project</a></li>
        </ul></li>


{% for item in pages %}
    <li class="dropdownactive">
      <a class="dropdown-toggle" data-toggle="dropdown" href="#">{{ item[1]['title'] }}</a>
      <ul class="dropdown-menu">
    {% for entry in item[1]['subs'] %}
        <li><a href="{{ entry[2] }}">{{ entry[1] }}</a></li>
    {% endfor %}
      </ul>
    </li>
{% endfor %}

</ul>
        </div>
      </div>
    </div>
</div>
